import tomllib
from pathlib import Path

import click
from notifypy import Notify

from backup_reminder.checker import (
    IncorrectBackupConfig,
    commit_backup,
    compute_last_backup_age_in_days,
    load_config,
    new_backup_is_needed,
)
from backup_reminder.generator import init_config

notification = Notify(
    default_notification_title="Backups status",
    default_application_name="Backup Checker",
    default_notification_icon=str(
        Path(__file__).parent.resolve() / "../logo.svg"
    ),
)

BACKUP_NEVER_PERFORMED_MESSAGE = (
    "You have never performed a backup. "
    "Please make one as soon as possible."
)

NO_BACKUP_NEEDED_MESSAGE = "No backup needed you're all good and safe."

NO_CONFIG_MESSAGE = (
    "No configuration file found. " "Please create one with the init command."
)

CONFIG_ERROR_MESSAGE = (
    "An error occured during configuration loading. "
    "Please check and fix your configuration file."
)

COMMIT_MESSAGE = "Your backup has been committed."


@click.group()
def cli():
    pass


@cli.command(help="Initialize the backup checker configuration.")
@click.option(
    "--backup-interval",
    default=5,
    help="""Amount of days between backups. A notification will be sent after
    this amount of days passed without any backup.""",
)
def init(backup_interval):
    click.echo("Generating configuration...")
    config_file = init_config(Path.home(), backup_interval)
    click.echo(f"Configuration generated in {config_file}")


@cli.command(help="Check if a new backup is necessary.")
def check():
    try:
        config = load_config(Path.home())
        backup_is_needed = new_backup_is_needed(config)

        if backup_is_needed:
            age = compute_last_backup_age_in_days(config)

            if age is None:
                message = BACKUP_NEVER_PERFORMED_MESSAGE
                click.echo(message)

                notification.message = message
                notification.title = "You need a new backup"
                notification.send()

                return

            message = (
                f"Your last backup was done {age} days ago. "
                "Please make a new one as soon as possible."
            )
            click.echo(message)

            notification.message = message
            notification.title = "You need a new backup"
            notification.send()

            return

        click.echo(NO_BACKUP_NEEDED_MESSAGE)
    except IncorrectBackupConfig:
        click.echo(CONFIG_ERROR_MESSAGE)
        exit(1)
    except FileNotFoundError:
        click.echo(NO_CONFIG_MESSAGE)
        exit(1)


@cli.command(help="Commit your backup.")
def commit():
    try:
        config = load_config(Path.home())

        if new_backup_is_needed(config):
            commit_backup(Path.home())
            click.echo(COMMIT_MESSAGE)
    except IncorrectBackupConfig:
        click.echo(CONFIG_ERROR_MESSAGE)
        exit(1)
    except FileNotFoundError:
        click.echo(NO_CONFIG_MESSAGE)
        exit(1)


@cli.command(help="Show bare CLI current version")
def version():
    with open(
        (Path(__file__).parent.resolve() / "../pyproject.toml"), "rb"
    ) as f:
        data = tomllib.load(f)
        click.echo(f"bare version {data["tool"]["poetry"]["version"]}")


if __name__ == "__main__":
    cli()



from pathlib import Path

from jinja2 import Environment, PackageLoader, select_autoescape

from backup_reminder.checker import LAST_FILE


def init_config(config_folder_path: str, backup_interval: int = 5) -> Path:
    config_file_path = Path(config_folder_path) / LAST_FILE

    with open(config_file_path, "w") as config_file:
        env = Environment(
            loader=PackageLoader("backup_reminder.generator"),
            autoescape=select_autoescape(),
            keep_trailing_newline=True,
        )

        template = env.get_template("last-backup.yaml.jinja")

        config_file.write(template.render(backup_interval=backup_interval))

    return config_file_path



import datetime
from pathlib import Path
from typing import Optional

import yaml

LAST_FILE = "last-backup.yaml"
LAST_BACKUP_KEY = "last_backup"
BACKUP_INTERVAL_KEY = "backup_interval"


class IncorrectBackupConfig(Exception):
    pass


class BackupConfig:
    def __init__(
        self,
        last_backup_date: Optional[datetime.datetime],
        backup_interval: int,
    ):
        self.last_backup_date = last_backup_date
        self.backup_interval = backup_interval


def load_config(backup_file_path: str) -> BackupConfig:
    file = Path(f"{backup_file_path}/{LAST_FILE}")
    with open(file, "r") as config_file:
        try:
            config = yaml.safe_load(config_file)

            if config is None:
                raise IncorrectBackupConfig()

            if LAST_BACKUP_KEY not in config:
                raise IncorrectBackupConfig()

            if BACKUP_INTERVAL_KEY not in config:
                raise IncorrectBackupConfig()

            backup_interval = None

            if BACKUP_INTERVAL_KEY in config:
                try:
                    backup_interval = int(config[BACKUP_INTERVAL_KEY])
                except TypeError:
                    raise IncorrectBackupConfig()

            last_backup_date = None

            if config[LAST_BACKUP_KEY] is not None:
                try:
                    last_backup_date = datetime.datetime.fromtimestamp(
                        config[LAST_BACKUP_KEY]
                    )

                except TypeError:
                    raise IncorrectBackupConfig()

            return BackupConfig(last_backup_date, backup_interval)

        except yaml.YAMLError:
            raise IncorrectBackupConfig(
                "Incorrect or corrupted file."
                f"Please check the content of {backup_file_path}"
            )


def compute_last_backup_age_in_days(config) -> Optional[int]:
    if config.last_backup_date is None:
        return None

    now = datetime.datetime.now()

    difference = now - config.last_backup_date

    return difference.days


def new_backup_is_needed(config) -> bool:
    if config.last_backup_date is None:
        return True

    age = compute_last_backup_age_in_days(config)

    return age > config.backup_interval


def commit_backup(config_folder_path: str):
    old_config = load_config(config_folder_path)

    config_file_path = Path(config_folder_path) / LAST_FILE

    with open(config_file_path, "w") as config_file:
        config = dict(
            last_backup=datetime.datetime.now().timestamp(),
            backup_interval=old_config.backup_interval,
        )

        yaml.dump(config, config_file)

    return config_file_path
